@@ -136,7 +136,7 @@ int blkid_set_tag(blkid_dev dev, const char *name,
 	if (!dev || !name)
 		return -BLKID_ERR_PARAM;
 
-	if (!(val = blkid_strndup(value, vlength)) && value)
+	if (value && !(val = blkid_strndup(value, vlength)))
 		return -BLKID_ERR_MEM;
 
 	/*
@@ -390,7 +390,7 @@ extern char *optarg;
 extern int optind;
 #endif
 
-void usage(char *prog)
+void __attribute__((__noreturn__)) usage(char *prog)
 {
 	fprintf(stderr, "Usage: %s [-f blkid_file] [-m debug_mask] device "
 		"[type value]\n",
