@@ -49,7 +49,9 @@
 #include <stdlib.h>
 #include <sys/syslog.h>
 #include <sys/sysmacros.h>
-#include <linux/major.h>
+#ifdef HAVE_LINUX_MAJOR_H
+# include <linux/major.h>
+#endif
 #include <netdb.h>
 #include <lastlog.h>
 #include <security/pam_appl.h>
@@ -66,7 +68,8 @@
 #include "strutils.h"
 #include "nls.h"
 #include "xalloc.h"
-#include "writeall.h"
+#include "all-io.h"
+#include "fileutils.h"
 
 #include "logindefs.h"
 
@@ -124,10 +127,26 @@ struct login_context {
  * This bounds the time given to login.  Not a define so it can
  * be patched on machines where it's too small.
  */
-static int timeout = LOGIN_TIMEOUT;
+static unsigned int timeout = LOGIN_TIMEOUT;
 static int child_pid = 0;
 static volatile int got_sig = 0;
 
+#ifdef LOGIN_CHOWN_VCS
+/* true if the filedescriptor fd is a console tty, very Linux specific */
+static int is_consoletty(int fd)
+{
+	struct stat stb;
+
+	if ((fstat(fd, &stb) >= 0)
+	    && (major(stb.st_rdev) == TTY_MAJOR)
+	    && (minor(stb.st_rdev) < 64)) {
+		return 1;
+	}
+	return 0;
+}
+#endif
+
+
 /*
  * Robert Ambrose writes:
  * A couple of my users have a problem with login processes hanging around
@@ -153,8 +172,8 @@ static void timedout(int sig __attribute__ ((__unused__)))
 {
 	signal(SIGALRM, timedout2);
 	alarm(10);
-	/* TRANSLATORS: The standard value for %d is 60. */
-	warnx(_("timed out after %d seconds"), timeout);
+	/* TRANSLATORS: The standard value for %u is 60. */
+	warnx(_("timed out after %u seconds"), timeout);
 	signal(SIGALRM, SIG_IGN);
 	alarm(0);
 	timedout2(0);
@@ -184,7 +203,7 @@ static void sig_handler(int signal)
  */
 static void __attribute__ ((__noreturn__)) sleepexit(int eval)
 {
-	sleep(getlogindefs_num("FAIL_DELAY", LOGIN_EXIT_TIMEOUT));
+	sleep((unsigned int)getlogindefs_num("FAIL_DELAY", LOGIN_EXIT_TIMEOUT));
 	exit(eval);
 }
 
@@ -300,15 +319,12 @@ static void chown_tty(struct login_context *cxt)
 
 	grname = getlogindefs_str("TTYGROUP", TTYGRPNAME);
 	if (grname && *grname) {
-		if (*grname >= 0 && *grname <= 9)		/* group by ID */
-			gid = getlogindefs_num("TTYGROUP", gid);
-		else {						/* group by name */
-			struct group *gr = getgrnam(grname);
-			if (gr)
-				gid = gr->gr_gid;
-		}
+		struct group *gr = getgrnam(grname);
+		if (gr)	/* group by name */
+			gid = gr->gr_gid;
+		else	/* group by ID */
+			gid = (gid_t) getlogindefs_num("TTYGROUP", gid);
 	}
-
 	if (fchown(0, uid, gid))				/* tty */
 		chown_err(cxt->tty_name, uid, gid);
 	if (fchmod(0, cxt->tty_mode))
@@ -316,15 +332,15 @@ static void chown_tty(struct login_context *cxt)
 
 #ifdef LOGIN_CHOWN_VCS
 	if (is_consoletty(0)) {
-		if (chown(cxt->vcs, uid, gid))			/* vcs */
-			chown_err(cxt->vcs, uid, gid);
-		if (chmod(cxt->vcs, cxt->tty_mode))
-			chmod_err(cxt->vcs, cxt->tty_mode);
+		if (chown(cxt->vcsn, uid, gid))			/* vcs */
+			chown_err(cxt->vcsn, uid, gid);
+		if (chmod(cxt->vcsn, cxt->tty_mode))
+			chmod_err(cxt->vcsn, cxt->tty_mode);
 
-		if (chown(cxt->vcsa, uid, gid))			/* vcsa */
-			chown_err(cxt->vcsa, uid, gid);
-		if (chmod(cxt->vcsa, cxt->tty_mode))
-			chmod_err(cxt->vcsa, cxt->tty_mode);
+		if (chown(cxt->vcsan, uid, gid))			/* vcsa */
+			chown_err(cxt->vcsan, uid, gid);
+		if (chmod(cxt->vcsan, cxt->tty_mode))
+			chmod_err(cxt->vcsan, cxt->tty_mode);
 	}
 #endif
 }
@@ -348,7 +364,7 @@ static void init_tty(struct login_context *cxt)
 	 *
 	 * More precisely, the problem is  ttyn := ttyname(0); ...; chown(ttyn);
 	 * here ttyname() might return "/tmp/x", a hardlink to a pseudotty.
-	 * All of this is a problem only when login is suid, which it isnt.
+	 * All of this is a problem only when login is suid, which it isn't.
 	 */
 	if (!cxt->tty_path || !*cxt->tty_path ||
 	    lstat(cxt->tty_path, &st) != 0 || !S_ISCHR(st.st_mode) ||
@@ -372,9 +388,11 @@ static void init_tty(struct login_context *cxt)
 	}
 
 #ifdef LOGIN_CHOWN_VCS
-	/* find names of Virtual Console devices, for later mode change */
-	snprintf(cxt->vcsn, sizeof(cxt->vcsn), "/dev/vcs%s", cxt->tty_number);
-	snprintf(cxt->vcsan, sizeof(cxt->vcsan), "/dev/vcsa%s", cxt->tty_number);
+	if (cxt->tty_number) {
+		/* find names of Virtual Console devices, for later mode change */
+		snprintf(cxt->vcsn, sizeof(cxt->vcsn), "/dev/vcs%s", cxt->tty_number);
+		snprintf(cxt->vcsan, sizeof(cxt->vcsan), "/dev/vcsa%s", cxt->tty_number);
+	}
 #endif
 
 	tcgetattr(0, &tt);
@@ -391,6 +409,14 @@ static void init_tty(struct login_context *cxt)
 	/* Kill processes left on this tty */
 	tcsetattr(0, TCSAFLUSH, &ttt);
 
+	/*
+	 * Let's close file decriptors before vhangup
+	 * https://lkml.org/lkml/2012/6/5/145
+	 */
+	close(STDIN_FILENO);
+	close(STDOUT_FILENO);
+	close(STDERR_FILENO);
+
 	signal(SIGHUP, SIG_IGN);	/* so vhangup() wont kill us */
 	vhangup();
 	signal(SIGHUP, SIG_DFL);
@@ -403,21 +429,6 @@ static void init_tty(struct login_context *cxt)
 }
 
 
-#ifdef LOGIN_CHOWN_VCS
-/* true if the filedescriptor fd is a console tty, very Linux specific */
-static int is_consoletty(int fd)
-{
-	struct stat stb;
-
-	if ((fstat(fd, &stb) >= 0)
-	    && (major(stb.st_rdev) == TTY_MAJOR)
-	    && (minor(stb.st_rdev) < 64)) {
-		return 1;
-	}
-	return 0;
-}
-#endif
-
 /*
  * Log failed login attempts in _PATH_BTMP if that exists.
  * Must be called only with username the name of an actual user.
@@ -507,7 +518,8 @@ static void log_lastlog(struct login_context *cxt)
 	if (fd < 0)
 		return;
 
-	lseek(fd, (off_t) cxt->pwd->pw_uid * sizeof(ll), SEEK_SET);
+	if (lseek(fd, (off_t) cxt->pwd->pw_uid * sizeof(ll), SEEK_SET) == -1)
+		return;
 
 	/*
 	 * Print last log message
@@ -573,24 +585,34 @@ static void log_utmp(struct login_context *cxt)
 	/* If we can't find a pre-existing entry by pid, try by line.
 	 * BSD network daemons may rely on this.
 	 */
-	if (utp == NULL) {
+	if (utp == NULL && cxt->tty_name) {
 		setutent();
 		ut.ut_type = LOGIN_PROCESS;
 		strncpy(ut.ut_line, cxt->tty_name, sizeof(ut.ut_line));
 		utp = getutline(&ut);
 	}
 
+	/* If we can't find a pre-existing entry by pid and line, try it by id.
+	 * Very stupid telnetd deamons don't set up utmp at all (kzak) */
+	if (utp == NULL && cxt->tty_number) {
+	     setutent();
+	     ut.ut_type = DEAD_PROCESS;
+	     strncpy(ut.ut_id, cxt->tty_number, sizeof(ut.ut_id));
+	     utp = getutid(&ut);
+	}
+
 	if (utp)
 		memcpy(&ut, utp, sizeof(ut));
 	else
 		/* some gettys/telnetds don't initialize utmp... */
 		memset(&ut, 0, sizeof(ut));
 
-	if (ut.ut_id[0] == 0)
+	if (cxt->tty_number && ut.ut_id[0] == 0)
 		strncpy(ut.ut_id, cxt->tty_number, sizeof(ut.ut_id));
-
-	strncpy(ut.ut_user, cxt->username, sizeof(ut.ut_user));
-	xstrncpy(ut.ut_line, cxt->tty_name, sizeof(ut.ut_line));
+	if (cxt->username)
+		strncpy(ut.ut_user, cxt->username, sizeof(ut.ut_user));
+	if (cxt->tty_name)
+		xstrncpy(ut.ut_line, cxt->tty_name, sizeof(ut.ut_line));
 
 #ifdef _HAVE_UT_TV		/* in <utmpbits.h> included by <utmp.h> */
 	gettimeofday(&tv, NULL);
@@ -623,6 +645,9 @@ static void log_syslog(struct login_context *cxt)
 {
 	struct passwd *pwd = cxt->pwd;
 
+	if (!cxt->tty_name)
+		return;
+
 	if (!strncmp(cxt->tty_name, "ttyS", 4))
 		syslog(LOG_INFO, _("DIALUP AT %s BY %s"),
 		       cxt->tty_name, pwd->pw_name);
@@ -761,7 +786,8 @@ static pam_handle_t *init_loginpam(struct login_context *cxt)
 
 static void loginpam_auth(struct login_context *cxt)
 {
-	int rc, failcount = 0, show_unknown, retries;
+	int rc, show_unknown;
+	unsigned int retries, failcount = 0;
 	const char *hostname = cxt->hostname ? cxt->hostname :
 			       cxt->tty_name ? cxt->tty_name : "<unknown>";
 	pam_handle_t *pamh = cxt->pamh;
@@ -797,7 +823,7 @@ static void loginpam_auth(struct login_context *cxt)
 			loginpam_get_username(pamh, &cxt->username);
 
 		syslog(LOG_NOTICE,
-		       _("FAILED LOGIN %d FROM %s FOR %s, %s"),
+		       _("FAILED LOGIN %u FROM %s FOR %s, %s"),
 		       failcount, hostname,
 		       cxt->username ? cxt->username : "(unknown)",
 		       pam_strerror(pamh, rc));
@@ -820,7 +846,7 @@ static void loginpam_auth(struct login_context *cxt)
 
 		if (rc == PAM_MAXTRIES)
 			syslog(LOG_NOTICE,
-			       _("TOO MANY LOGIN TRIES (%d) FROM %s FOR %s, %s"),
+			       _("TOO MANY LOGIN TRIES (%u) FROM %s FOR %s, %s"),
 			       failcount, hostname,
 			       cxt->username ? cxt->username : "(unknown)",
 			       pam_strerror(pamh, rc));
@@ -1006,13 +1032,17 @@ static int get_hushlogin_status(struct passwd *pwd)
 			gid_t egid = getegid();
 
 			sprintf(buf, "%s/%s", pwd->pw_dir, file);
-			setregid(-1, pwd->pw_gid);
-			setreuid(0, pwd->pw_uid);
-			ok = effective_access(buf, O_RDONLY) == 0;
-			setuid(0);	/* setreuid doesn't do it alone! */
-			setreuid(ruid, 0);
-			setregid(-1, egid);
 
+			if (setregid(-1, pwd->pw_gid) == 0 &&
+			    setreuid(0, pwd->pw_uid) == 0)
+				ok = effective_access(buf, O_RDONLY) == 0;
+
+			if (setuid(0) != 0 ||
+			    setreuid(ruid, 0) != 0 ||
+			    setregid(-1, egid) != 0) {
+				syslog(LOG_ALERT, _("hush login status: restore original IDs failed"));
+				exit(EXIT_FAILURE);
+			}
 			if (ok)
 				return 1;	/* enabled by user */
 		}
@@ -1101,7 +1131,7 @@ static void fork_session(struct login_context *cxt)
 		exit(EXIT_FAILURE);
 
 	/*
-	 * Problem: if the user's shell is a shell like ash that doesnt do
+	 * Problem: if the user's shell is a shell like ash that doesn't do
 	 * setsid() or setpgrp(), then a ctrl-\, sending SIGQUIT to every
 	 * process in the pgrp, will kill us.
 	 */
@@ -1141,17 +1171,15 @@ static void init_environ(struct login_context *cxt)
 	}
 
 	setenv("HOME", pwd->pw_dir, 0);	/* legal to override */
+	setenv("USER", pwd->pw_name, 1);
 	setenv("SHELL", pwd->pw_shell, 1);
 	setenv("TERM", termenv, 1);
 
 	if (pwd->pw_uid)
-		setenv("PATH", getlogindefs_str("ENV_PATH", _PATH_DEFPATH), 1);
-	else {
-		const char *x = getlogindefs_str("ENV_ROOTPATH", NULL);
-		if (!x)
-			x = getlogindefs_str("ENV_SUPATH", _PATH_DEFPATH_ROOT);
-		setenv("PATH", x, 1);
-	}
+		logindefs_setenv("PATH", "ENV_PATH", _PATH_DEFPATH);
+
+	else if (logindefs_setenv("PATH", "ENV_ROOTPATH", NULL) != 0)
+		logindefs_setenv("PATH", "ENV_SUPATH", _PATH_DEFPATH_ROOT);
 
 	/* mailx will give a funny error msg if you forget this one */
 	len = snprintf(tmp, sizeof(tmp), "%s/%s", _PATH_MAILDIR, pwd->pw_name);
@@ -1226,11 +1254,11 @@ int main(int argc, char **argv)
 		.conv = { misc_conv, NULL }	/* PAM conversation function */
 	};
 
-	timeout = getlogindefs_num("LOGIN_TIMEOUT", LOGIN_TIMEOUT);
+	timeout = (unsigned int)getlogindefs_num("LOGIN_TIMEOUT", LOGIN_TIMEOUT);
 
 	signal(SIGALRM, timedout);
 	siginterrupt(SIGALRM, 1);	/* we have to interrupt syscalls like ioclt() */
-	alarm((unsigned int)timeout);
+	alarm(timeout);
 	signal(SIGQUIT, SIG_IGN);
 	signal(SIGINT, SIG_IGN);
 
@@ -1291,7 +1319,7 @@ int main(int argc, char **argv)
 			*p++ = ' ';
 	}
 
-	for (cnt = getdtablesize(); cnt > 2; cnt--)
+	for (cnt = get_fd_tabsize() - 1; cnt > 2; cnt--)
 		close(cnt);
 
 	setpgrp();	 /* set pgid to pid this means that setsid() will fail */
@@ -1310,7 +1338,7 @@ int main(int argc, char **argv)
 	/*
 	 * Authentication may be skipped (for example, during krlogin, rlogin,
 	 * etc...), but it doesn't mean that we can skip other account checks.
-	 * The account could be disabled or password expired (althought
+	 * The account could be disabled or password expired (although
 	 * kerberos ticket is valid).         -- kzak@redhat.com (22-Feb-2006)
 	 */
 	loginpam_acct(&cxt);
